@@ -1,10 +1,16 @@
-use crate::api::{Handle, ManagedTypeApi};
+use crate::api::{Handle, ManagedTypeApi, ManagedTypeApiImpl};
 
-impl ManagedTypeApi for super::UncallableApi {
-    fn instance() -> Self {
+use super::UncallableApi;
+
+impl ManagedTypeApi for UncallableApi {
+    type ManagedTypeApiImpl = Self;
+
+    fn managed_type_impl() -> Self::ManagedTypeApiImpl {
         unreachable!()
     }
+}
 
+impl ManagedTypeApiImpl for UncallableApi {
     fn mb_to_big_int_unsigned(&self, _buffer_handle: Handle) -> Handle {
         unreachable!()
     }
