@@ -1,5 +1,5 @@
 use crate::{
-    api::{ErrorApi, ManagedTypeApi},
+    api::{ErrorApi, ErrorApiImpl, ManagedTypeApi},
     err_msg,
     types::ManagedBytesTopDecodeInput,
     DynArgInput, HexCallDataDeserializer,
@@ -10,15 +10,15 @@ where
     A: ManagedTypeApi + ErrorApi,
 {
     deser: HexCallDataDeserializer<'a>,
-    api: A,
+    _api: A,
 }
 
 impl<'a, A> CallDataArgLoader<'a, A>
 where
     A: ManagedTypeApi + ErrorApi,
 {
-    pub fn new(deser: HexCallDataDeserializer<'a>, api: A) -> Self {
-        CallDataArgLoader { deser, api }
+    pub fn new(deser: HexCallDataDeserializer<'a>, _api: A) -> Self {
+        CallDataArgLoader { deser, _api }
     }
 }
 
@@ -28,12 +28,7 @@ where
 {
     type ItemInput = ManagedBytesTopDecodeInput<A>;
 
-    type ErrorApi = A;
-
-    #[inline]
-    fn dyn_arg_vm_api(&self) -> Self::ErrorApi {
-        self.api.clone()
-    }
+    type ManagedTypeErrorApi = A;
 
     #[inline]
     fn has_next(&self) -> bool {
@@ -42,13 +37,9 @@ where
 
     fn next_arg_input(&mut self) -> ManagedBytesTopDecodeInput<A> {
         match self.deser.next_argument() {
-            Ok(Some(arg_bytes)) => {
-                ManagedBytesTopDecodeInput::new(self.api.clone(), arg_bytes.into())
-            },
-            Ok(None) => self
-                .dyn_arg_vm_api()
-                .signal_error(err_msg::ARG_WRONG_NUMBER),
-            Err(sc_err) => self.dyn_arg_vm_api().signal_error(sc_err.as_bytes()),
+            Ok(Some(arg_bytes)) => ManagedBytesTopDecodeInput::<A>::new(arg_bytes.into()),
+            Ok(None) => A::error_api_impl().signal_error(err_msg::ARG_WRONG_NUMBER),
+            Err(sc_err) => A::error_api_impl().signal_error(sc_err.as_bytes()),
         }
     }
 }
