@@ -4,6 +4,7 @@
 #![feature(control_flow_enum)]
 #![allow(clippy::type_complexity)]
 #![allow(deprecated)]
+#![feature(generic_associated_types)]
 
 pub use elrond_wasm_derive::{self as derive, contract, module, proxy};
 
@@ -21,6 +22,8 @@ pub mod api;
 pub mod contract_base;
 pub mod err_msg;
 pub mod esdt;
+pub mod external_view_contract;
+pub mod formatter;
 pub mod hex_call_data;
 pub mod hex_util;
 pub mod io;
