@@ -194,7 +194,7 @@ mod tests {
         assert!(sc_result_ok.unwrap() == 5);
 
         let result_err: Result<i32, DecodeError> =
-            Result::Err(DecodeError::from(&b"Decode Error"[..]).into());
+            Result::Err(DecodeError::from("Decode Error").into());
         let sc_result_err: SCResult<i32> = result_err.into();
 
         assert!(sc_result_err.err().unwrap().as_bytes() == &b"Decode Error"[..]);
