@@ -272,7 +272,10 @@ impl<M: ManagedTypeApi> TopDecode for ManagedBuffer<M> {
 
 impl<M: ManagedTypeApi> NestedDecode for ManagedBuffer<M> {
     fn dep_decode<I: NestedDecodeInput>(input: &mut I) -> Result<Self, DecodeError> {
-        input.read_specialized((), |_| Err(DecodeError::UNSUPPORTED_OPERATION))
+        input.read_specialized((), |input| {
+            let boxed_bytes = BoxedBytes::dep_decode(input)?;
+            Ok(Self::new_from_bytes(boxed_bytes.as_slice()))
+        })
     }
 
     fn dep_decode_or_exit<I: NestedDecodeInput, ExitCtx: Clone>(
@@ -280,8 +283,9 @@ impl<M: ManagedTypeApi> NestedDecode for ManagedBuffer<M> {
         c: ExitCtx,
         exit: fn(ExitCtx, DecodeError) -> !,
     ) -> Self {
-        input.read_specialized_or_exit((), c, exit, |_, c| {
-            exit(c, DecodeError::UNSUPPORTED_OPERATION)
+        input.read_specialized_or_exit((), c, exit, |input, c| {
+            let boxed_bytes = BoxedBytes::dep_decode_or_exit(input, c, exit);
+            Self::new_from_bytes(boxed_bytes.as_slice())
         })
     }
 }
