@@ -148,10 +148,7 @@ where
         C: TryStaticCast,
         F: FnOnce(&mut Self) -> Result<T, DecodeError>,
     {
-        if let Some(result) = self.buffer.type_manager().try_cast_ref::<T>() {
-            // API for instancing empty Vec
-            Ok(result.clone())
-        } else if let Some(result) = try_execute_then_cast(|| {
+        if let Some(result) = try_execute_then_cast(|| {
             if let Some(mb_context) = context.try_cast_ref::<ManagedBufferSizeContext>() {
                 self.read_managed_buffer_of_size(mb_context.0)
             } else {
@@ -182,10 +179,7 @@ where
         F: FnOnce(&mut Self, ExitCtx) -> T,
         ExitCtx: Clone,
     {
-        if let Some(result) = self.buffer.type_manager().try_cast_ref::<T>() {
-            // API for instancing empty Vec
-            result.clone()
-        } else if let Some(result) = try_execute_then_cast(|| {
+        if let Some(result) = try_execute_then_cast(|| {
             if let Some(mb_context) = context.try_cast_ref::<ManagedBufferSizeContext>() {
                 self.read_managed_buffer_of_size_or_exit(mb_context.0, c.clone(), exit)
             } else {
