@@ -2,7 +2,7 @@ use alloc::boxed::Box;
 use elrond_codec::{try_execute_then_cast, DecodeError, TopDecodeInput, TryStaticCast};
 
 use crate::{
-    api::ManagedTypeApi,
+    api::{ErrorApiImpl, ManagedTypeApi},
     types::{BigInt, BigUint, ManagedBuffer},
 };
 
@@ -26,7 +26,7 @@ where
         if let Some(num) = self.parse_as_u64() {
             num
         } else {
-            M::instance().signal_error(DecodeError::INPUT_TOO_LONG.message_bytes())
+            M::error_api_impl().signal_error(DecodeError::INPUT_TOO_LONG.message_bytes())
         }
     }
 
